@@ -1,10 +1,12 @@
 # -*- coding: utf-8 -*-
-# Pretend to be /usr/bin/id3v2 from id3lib, sort of.
 # Copyright 2005 Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of version 2 of the GNU General Public License as
-# published by the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+"""Pretend to be /usr/bin/id3v2 from id3lib, sort of."""
 
 import sys
 import codecs
@@ -15,8 +17,8 @@ from optparse import SUPPRESS_HELP
 import mutagen
 import mutagen.id3
 from mutagen.id3 import Encoding, PictureType
-from mutagen._senf import fsnative, print_, argv, fsn2text, text2fsn, \
-    fsn2bytes, bytes2fsn
+from mutagen._senf import fsnative, print_, argv, fsn2text, fsn2bytes, \
+    bytes2fsn
 from mutagen._compat import PY2, text_type
 
 from ._util import split_escape, SignalHandler, OptionParser
@@ -116,9 +118,7 @@ def frame_from_fsnative(arg):
 
     assert isinstance(arg, fsnative)
 
-    text = fsn2text(arg)
-    if arg != text2fsn(text):
-        raise ValueError("%r can't be decoded" % arg)
+    text = fsn2text(arg, strict=True)
     if PY2:
         return text.encode("ascii")
     else:
@@ -140,10 +140,7 @@ def value_from_fsnative(arg, escape):
             bytes_ = codecs.escape_decode(bytes_)[0]
         arg = bytes2fsn(bytes_, "utf-8")
 
-    text = fsn2text(arg)
-    # only allow values which can be converted back
-    if arg != text2fsn(text):
-        raise ValueError("%r can't be decoded" % arg)
+    text = fsn2text(arg, strict=True)
     return text
 
 
@@ -286,6 +283,19 @@ def write_files(edits, filenames, escape):
                         frame = mutagen.id3.COMM(
                             encoding=3, text=value, lang=lang, desc=desc)
                         id3.add(frame)
+                elif frame == "USLT":
+                    for value in vlist:
+                        values = string_split(value, ":")
+                        if len(values) == 1:
+                            value, desc, lang = values[0], "", "eng"
+                        elif len(values) == 2:
+                            desc, value, lang = values[0], values[1], "eng"
+                        else:
+                            value = ":".join(values[1:-1])
+                            desc, lang = values[0], values[-1]
+                        frame = mutagen.id3.USLT(
+                            encoding=3, text=value, lang=lang, desc=desc)
+                        id3.add(frame)
                 elif frame == "UFID":
                     for value in vlist:
                         values = string_split(value, ":")
@@ -433,7 +443,8 @@ def main(argv):
         if (issubclass(frame, mutagen.id3.TextFrame)
                 or issubclass(frame, mutagen.id3.UrlFrame)
                 or issubclass(frame, mutagen.id3.POPM)
-                or frame in (mutagen.id3.APIC, mutagen.id3.UFID)):
+                or frame in (mutagen.id3.APIC, mutagen.id3.UFID,
+                             mutagen.id3.USLT)):
             parser.add_option(
                 "--" + key, action="callback", help=SUPPRESS_HELP,
                 type='string', metavar="value",  # optparse blows up with this
