@@ -434,6 +434,7 @@ class TextFrame(Frame):
         for val in other[:]:
             if val not in self:
                 self.append(val)
+                self.encoding = max(self.encoding, other.encoding)
         return self
 
     def _pprint(self):
@@ -669,6 +670,14 @@ class MVI(MVIN):
     "iTunes Movement Number/Count"
 
 
+class GRP1(TextFrame):
+    "iTunes Grouping"
+
+
+class GP1(GRP1):
+    "iTunes Grouping"
+
+
 class TDOR(TimeStampTextFrame):
     "Original Release Time"
 
@@ -1065,6 +1074,9 @@ class USLT(Frame):
 
     __hash__ = Frame.__hash__
 
+    def _pprint(self):
+        return "%s=%s=%s" % (self.desc, self.lang, self.text)
+
 
 @swap_to_string
 class SYLT(Frame):
